''  EInk Driver Test
''
''
'' Symbol       Designation     Input/Output            Logic                   Function
'' ---------------------------------------------------------------------------------------
''  H_DS        Module Select   Input                   L                       Data strobe
''  H_RW        Read/Write      Input                   H:Read/L:Write          Indicates the data bus direction
''  H_D[0..7]   Data            3-State Input/Output                            D:(7:MSB, 0:LSB)
''  H_CD        Command/Data    Input                   H:Command/L:Data        Swithcing command or data
''  H_ACK       Acknowledge     3-state Output          L                       Acknowledge
''  H_WUP       WakeUp          Input                   H                       Return from stand by mode
''  H_NRST      Reset           Input                   L                       Reset (Asynch)
'' 
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'' Module = Display driver (Trinity)
'' HOST   = Propeller

CON
  H_NRST        = 10
  H_WUP         = 11
  H_CD          = 12
  H_DS          = 13
  H_RW          = 14
  H_ACK         = 15

  EEProm_01         = %10100000
  EEProm_02         = %10100100
  EEProm_03         = %10100110
  EEProm_04         = %10101000
  EEProm_05         = %10101100
  EEProm_06         = %10101110

  #0, CMD_NONE, CMD_SHOW_PIC, CMD_SHOW_PARTIAL_PIC, CMD_CLR_WHITE, CMD_CLR_BLACK, CMD_INIT, CMD_DPT_BW, CMD_OR_90, CMD_MANUAL_REFRESH, CMD_INVERT, DRAW_STARTLOAD, DRAW_SENDTILE, DRAW_STOPLOAD
  #0, OBJ_TEXT, OBJ_PIXEL, OBJ_LINE, OBJ_ARC, OBJ_BITMAP, OBJ_CLR
  MaxObjects    = 50
  VideoBufferSize=20000
OBJ
  fntArial48 : "fntArial48"
  eeprom : "NS_eeprom_drv_011"
  'dbg  : "PC_interface"
  objects[MaxObjects]:"object"    
VAR
  long cogon, cog
  
  long command
  long address
  long x1,y1,x2,y2, img_size
  byte Videobuffer[VideoBufferSize]  
  byte buffer[4096]
  byte objCount
  

PUB Start|c
  
  repeat c from 0 to MaxObjects-1
    objects[c].Clean  



  'dbg.start(31,30)
  objCount:=0
  Stop
  
  'bytefill(@Videobuffer,$00,VideoBufferSize)      
  return cogon := (cog := cognew(@entry,@command)) > 0
  
   

PUB GetCog
  return cog  
PUB Stop

'' Stops the eink driver. Frees a cog.
  eeprom.Stop
  if cogon~
    cogstop(cog)

  command := CMD_NONE

'PUB ShowPicture(arg_address)

'  address:= arg_address
'  command:= CMD_SHOW_PIC

PUB ShowPartialPicture(arg_address,arg_x1,arg_y1,arg_x2,arg_y2)
  
  address:= arg_address
  x1:=arg_x1
  y1:=arg_y1
  x2:=arg_x2
  y2:=arg_y2
  img_size:=(x2-x1) * (y2-y1)
  command:= CMD_SHOW_PARTIAL_PIC
    
PUB CLR_White
  command:= CMD_CLR_WHITE

PUB NegativePicture
  command:=CMD_INVERT   

'PUB ManualRefresh
'  command:= CMD_MANUAL_REFRESH
  
PUB CLR_Black
  command:= CMD_CLR_BLACK   

PUB IsDone

  return (command == CMD_NONE)

PUB InitDisplay

  command:= CMD_INIT

PUB Portrait
  command := CMD_OR_90

PUB SetDepthBW
  command:= CMD_DPT_BW

   
PUB StringArialChar(x,y,str, startY, endY, minY) | w, h, s, z,  ch', startY, endY
  
  w:= fntArial48.GetW(str) 
  h:= fntArial48.GetH(str) 
  s:= fntArial48.GetStart(str)
  z:= fntArial48.GetSize(str)

 '  startY:=0
 '  endY:=h-1

  if s>0 
    w:=w/8
    x:=x/8
'    if y<minY
 '     startY:=minY-y
'    if (y+h)>maxY
 '     endY:= maxY-y
    bytefill(@buffer,0,4096)  
'    eeprom.read(@buffer, s,z)
    eeprom.read(@buffer, s+(startY*w),w*(endY-startY+1))
    repeat until eeprom.IsDone

    y:=(y-minY)*100+x+@Videobuffer
   ' y:=y*100
   ' y:=y+x
   ' y:=y+@Videobuffer

    repeat ch from 0 to endY-startY+1
      bytemove(y+100*(ch+startY),@buffer+(ch*w),w)

      
PUB StringArial(x,y,str,minY,maxY) | c,xc, tChar,  startY, endY, size

  eeprom.start(8, 9, 0, EEProm_06) 
  'xc:=x
    
  size:=72
   
 
  startY:=0
  endY:=size-1
 

  if y<minY
     startY:=minY-y
  if (y+size)>maxY
     endY:= maxY-y

'  y:=y-minY
'  y:=y*100
'  y:=y+x
'  y:=y+@Videobuffer    
  c := str
                       
  repeat strsize(str)
      tChar:= byte[c++]
      StringArialChar(x,y,tChar,startY,endY, minY)
      x:=x+fntArial48.GetW(tChar)
      

    'ShowPartialPicture(@Videobuffer,x,y,x+totW,y+maxH-1)
    'repeat until IsDone 
    

   'dbg.stop
  eeprom.Stop
PUB Text(x,y,strtext, size)

  if objCount < MaxObjects

    objects[objCount].SetObjectType(OBJ_TEXT)
    objects[objCount].SetDimension(x,y,x,y+size)   '' x2 not really necessary
    objects[objCount].SetBodyText(strtext)
    objCount++

PUB Draw | tile, objCnt, minY, maxY
  
  address:=@Videobuffer
  img_size:=VideoBufferSize

  minY:=0
  maxY:=VideoBufferSize/100-1
  DRAW_LoadBegin
  repeat until IsDone
  'dbg.dec(objCount)
  'dbg.out(13)
  repeat tile from 1 to (600 / (VideoBufferSize/100))
    bytefill(@Videobuffer,$00,VideoBufferSize)  
    repeat objCnt from 0 to objCount-1

       if (objects[objCnt].GetObjectType==OBJ_TEXT)  AND (NOT((objects[objCnt].GetY1=>maxY) or (objects[objCnt].GetY2=<minY)))                         
           'Draw_Text(objects[objCnt].GetX1,objects[objCnt].GetY1,objects[objCnt].GetBodyText,objects[objCnt].GetTextH,minY,maxY)
           Draw_Text(objects[objCnt].GetX1,objects[objCnt].GetY1,objects[objCnt].GetBodyText,minY,maxY)
           'dbg.out(13)
           'dbg.str(objects[objCnt].GetBodyText)
       if (objects[objCnt].GetObjectType==OBJ_CLR)  AND (NOT((objects[objCnt].GetY1=>maxY) or (objects[objCnt].GetY2=<minY)))                         
           'Draw_Text(objects[objCnt].GetX1,objects[objCnt].GetY1,objects[objCnt].GetBodyText,objects[objCnt].GetTextH,minY,maxY)
           Draw_Text(0,0,string(" "),minY,maxY)

    DRAW_Tile
    repeat until IsDone
    
    'bytefill(@Videobuffer,$00,VideoBufferSize)
    minY:=minY+(VideoBufferSize/100)
    maxY:=maxY+(VideoBufferSize/100)    
  '' --- END TILE LOOP --------------

  DRAW_LoadEnd
  repeat until IsDone
PUB Draw_Text(x,y,txt,minY,maxY)

  StringArial(x,y,txt,minY,maxY)

   
PUB DRAW_LoadBegin
  command:= DRAW_STARTLOAD
PUB DRAW_Tile
  command:= DRAW_SENDTILE  
PUB DRAW_LoadEnd
  command:= DRAW_STOPLOAD  


PUB BitMap(eepromAddr, startAddr, size,btx1,bty1,btx2,bty2)
  eeprom.start(8, 9, 0, eepromAddr)
  eeprom.read(@buffer, startAddr, size)
  repeat until eeprom.IsDone
  eeprom.stop 
  ShowPartialPicture(@buffer+1,btx1,bty1,btx2,bty2)
  repeat until IsDone

PUB Clr | tile

  DRAW_LoadBegin
  repeat until IsDone
  repeat tile from 1 to (600 / (VideoBufferSize/100))
    bytefill(@Videobuffer,$00,VideoBufferSize)  
    DRAW_Tile
    repeat until IsDone
  DRAW_LoadEnd
  repeat until IsDone

DAT
                                  org 0
entry

'  --------- Debugger Kernel add this at Entry (Addr 0) ---------
'   long $34FC1202,$6CE81201,$83C120B,$8BC0E0A,$E87C0E03,$8BC0E0A
'   long $EC7C0E05,$A0BC1207,$5C7C0003,$5C7C0003,$7FFC,$7FF8
'  -------------------------------------------------------------- 
                     'mov     init_par,par
                     call    #init

wait_for_command
                    mov     timer,param_idle_wait         'init timer
                    add     timer,cnt
wait_for_command_loop
                        
                    rdlong  temp2,m_command           wz  'check for a command
                    add     timer,#40                     'add max clocks this loop may take to timer
        if_z        waitcnt timer,param_idle_wait         'if no command, sleep before checking again
        if_z        jmp     #wait_for_command_loop

                    '---- Run Command ----
'                    cmp     temp2,#CMD_SHOW_PIC         wz
'        if_e        call    #show_pic
'        if_e        jmp     #command_done

                    cmp     temp2,#CMD_INIT             wz
        if_e        call    #init_pvi
        if_e        jmp     #command_done

                    cmp     temp2,#CMD_DPT_BW           wz
        if_e        call    #set_depth_BW
        if_e        jmp     #command_done

                    cmp     temp2,#CMD_CLR_WHITE        wz
        if_e        call    #erase_disp_WT               
        if_e        jmp     #command_done

                    cmp     temp2,#CMD_CLR_BLACK        wz
        if_e        call    #erase_disp_BK               
        if_e        jmp     #command_done

                    cmp     temp2,#CMD_SHOW_PARTIAL_PIC    wz
        if_e        call    #show_part_pic               
        if_e        jmp     #command_done

'                    cmp     temp2,#CMD_MANUAL_REFRESH    wz
'        if_e        call    #show_part_pic               
'        if_e        jmp     #command_done

                    cmp     temp2,#CMD_INVERT        wz
        if_e        call    #invert               
        if_e        jmp     #command_done
        
                    cmp     temp2,#CMD_OR_90        wz
        if_e        call    #set_orient_90               
        if_e        jmp     #command_done

''------------------------------------------------------------------
                    cmp     temp2,#DRAW_STARTLOAD        wz
        if_e        call    #DRW_STRLOAD               
        if_e        jmp     #command_done


                    cmp     temp2,#DRAW_SENDTILE         wz
        if_e        call    #DRW_SENDTILE               
        if_e        jmp     #command_done


                    cmp     temp2,#DRAW_STOPLOAD        wz
        if_e        call    #DRW_STPLOAD               
        if_e        jmp     #command_done

                    '---- Signal That The Command Is Done ----
command_done
                    wrlong  zero,m_command
                    jmp     #wait_for_command
                    
invert
                   ' mov m_DataByte, #0              
                    mov m_DataByte, m_NegativePicture                                        
                    call #writebyteCMD
invert_ret          ret

erase_disp_WT
                    'mov m_DataByte, #0              
                    mov m_DataByte, m_EraseDisplay                                        
                    call #writebyteCMD
                    'mov m_DataByte, #0                
                    mov m_DataByte, m_CLR_White                                          
                    call #writebyteDTA
erase_disp_WT_ret   ret

erase_disp_BK
                    'mov m_DataByte, #0              
                    mov m_DataByte, m_EraseDisplay                                        
                    call #writebyteCMD
                    'mov m_DataByte, #0                
                    mov m_DataByte, m_CLR_Black                                          
                    call #writebyteDTA
erase_disp_BK_ret   ret

init_pvi
                    or   OUTA, m_DS
                    or   OUTA, m_RW
                    andn OUTA, m_CD                                          
                    or   DIRA, m_OutPins

                    
                    andn OUTA, m_DataPins          
                    or   OUTA, m_NRST
                    'mov m_DataByte, #0              
                    mov m_DataByte, m_InitDisplay                                        
                    call #writebyteCMD

                    mov m_DataByte, #0                                              
                    call #writebyteDTA
init_pvi_ret        ret
set_depth_GR
                    'mov m_DataByte, #0            
                    mov m_DataByte, m_SetDepth                                      
                    call #writebyteCMD
                    'mov m_DataByte, #0            
                    mov m_DataByte, m_Depth_GreyScale                                     
                    call #writebyteDTA
set_depth_GR_ret   ret

set_depth_BW
                    'mov m_DataByte, #0            
                    mov m_DataByte, m_SetDepth                                      
                    call #writebyteCMD
                    'mov m_DataByte, #0            
                    mov m_DataByte, m_Depth_BlackWhite                                     
                    call #writebyteDTA
set_depth_BW_ret   ret

set_orient_90
                    'mov m_DataByte, #0             
                    mov m_DataByte, m_Orientation                                        
                    call #writebyteCMD
                    nop
                    'mov m_DataByte, #0                 
                    mov m_DataByte, m_Orientation_180                                          
                    call #writebyteDTA

set_orient_90_ret  ret                     

'manual_refresh
'                                mov timer, cnt
'                                add timer, #T_Ack                           
'                                waitcnt timer, #T_Ack                        
'                     'mov        m_DataByte, #0             
'                     mov        m_DataByte, m_ManualRefresh
'                     call       #writebyteCMD
'                     
'manual_refresh_ret ret


show_part_pic
                     
                     call       #Init
                     mov        m_DataByte, #0             
                     mov        m_DataByte, m_LoadPartialPicture                        
                     call       #writebyteCMD
                     
                     mov     temp_coo, m_X1
                     shr        temp_coo, #8
                     'mov        m_DataByte, #0
                     mov        m_DataByte,temp_coo
                     call       #flip
                     call       #writebyteDTA

                     mov     temp_coo, m_X1
                     'mov        m_DataByte, #0                     
                     mov        m_DataByte,temp_coo
                     call       #flip
                     call       #writebyteDTA

                     mov     temp_coo, m_Y1
                     shr        temp_coo, #8
                     'mov        m_DataByte, #0
                     mov        m_DataByte,temp_coo
                     call       #flip
                     call       #writebyteDTA

                     mov     temp_coo, m_Y1
                     'mov        m_DataByte, #0                     
                     mov        m_DataByte,temp_coo
                     call       #flip
                     call       #writebyteDTA

                     mov     temp_coo, m_X2
                     shr        temp_coo, #8
                     'mov        m_DataByte, #0
                     mov        m_DataByte,temp_coo
                     call       #flip
                     call       #writebyteDTA

                     mov     temp_coo, m_X2
                     'mov        m_DataByte, #0                     
                     mov        m_DataByte,temp_coo
                     call       #flip
                     call       #writebyteDTA
                     
                     mov     temp_coo, m_Y2
                     shr        temp_coo, #8
                     'mov        m_DataByte, #0
                     mov        m_DataByte,temp_coo
                     call       #flip
                     call       #writebyteDTA

                     mov     temp_coo, m_Y2
                     'mov        m_DataByte, #0                     
                     mov        m_DataByte,temp_coo
                     call       #flip
                     call       #writebyteDTA   

                      
                     mov m_Count, m_image_size
                     'add m_Address, #1
      :loopData1     
                     
                     rdbyte m_DataByte, m_Address
                     'shr    m_DataByte, #24
                     'mov m_DataByte, #%11110000
                     call #flip
                     call #writebyteDTA
                     
                     add  m_Address, #1

                     djnz m_Count, #:loopData1

                     'mov m_DataByte, #0
                     mov m_DataByte, m_StopLoading   
                     call #writebyteCMD              

                     'mov m_DataByte, #0                                
                     mov m_DataByte, m_DisplayPartialPicture  
                     call #writebyteCMD                     

show_part_pic_ret  ret

DRW_STRLoad          'call #Init
                     'mov  m_DataByte, #0             
                     mov  m_DataByte, m_LoadPicture                        
                     call #writebyteCMD
DRW_STRLoad_ret ret

DRW_SendTile
                     call #Init   
                     mov m_Count, m_image_size
      :loopDataDRW     
                     
                     rdbyte m_DataByte, m_Address
                     'shr    m_DataByte, #24
                     'mov m_DataByte, #%11110000
                     call #flip
                     call #writebyteDTA
                     
                     add  m_Address, #1

                     djnz m_Count, #:loopDataDRW
DRW_SendTile_ret ret

DRW_STPLoad
                     mov m_DataByte, m_StopLoading   
                     call #writebyteCMD              
                     mov m_DataByte, m_DisplayPicture  
                     call #writebyteCMD
DRW_STPLoad_ret ret
flip
                     rev m_DataByte, #24
flip_ret ret


'show_pic             call #Init
'                     'mov  m_DataByte, #0             
'                     mov  m_DataByte, m_LoadPicture                        
'                     call #writebyteCMD
'                     'mov temp2, m_Address
'                     'mov  m_AddressTemp, temp2
'      :loopData                                                    
'                     rdlong m_DataByte, m_Address
'                     shr  m_DataByte, #24
'                     call #flip
'                     call #writebyteDTA
'                     add  m_Address, #1
'                     djnz m_Count, #:loopData
'                     nop
'                     'mov m_DataByte, #0
'                     mov m_DataByte, m_StopLoading   
'                     call #writebyteCMD              
'                     nop
'                     nop
'                    nop
'                    'mov m_DataByte, #0                                
'                     mov m_DataByte, m_DisplayPicture  
'                     call #writebyteCMD
'show_pic_ret         ret



init
                     mov    m_Command, par          ' get the HUB address of command
                     mov    temp, par
                     add    temp, #4
                     rdlong    m_Address, temp         ' get the HUB address of the image
                     add    temp, #4
                     rdlong    m_X1, temp              ' get coordinate X1
                     add    temp, #4
                     rdlong    m_Y1, temp              ' get coordinate Y1
                     add    temp, #4
                     rdlong    m_X2, temp              ' get coordinate X2
                     add    temp, #4
                     rdlong    m_Y2, temp              ' get coordinate Y2
                     add    temp, #4
                     rdlong m_image_size, temp
                     
init_ret  ret
writebyteCMD                    'or   OUTA, m_DS              
                                andn OUTA, m_DataPins         
                                shl  m_DataByte, #16         
                                or   OUTA, m_DataByte         
                         
                                andn OUTA, m_RW               
                                or   OUTA, m_CD
                                              
                                andn OUTA, m_DS

                                waitpeq m_ACK,m_ACK
                                waitpne m_ACK, m_ACK 

                                or   OUTA,m_DS               

writebyteCMD_ret ret                               


writebyteDTA                               
                                andn OUTA, m_RW              
                                andn OUTA, m_CD
                                andn OUTA, m_DataPins         
                                shl  m_DataByte, #16         
                                or   OUTA, m_DataByte         
                                andn OUTA, m_DS               

                                waitpeq m_ACK,m_ACK
                                waitpne m_ACK, m_ACK 
                                or   OUTA,m_DS                

writebyteDTA_ret ret
'------------------------------------------------- VARS ----------------------------------------------------
m_OutPins    long %00000000111111110111110000000000
m_DataPins   long %00000000111111110000000000000000



m_LoadPicture           long  %00000101
m_StopLoading           long  %10000101
m_DisplayPicture        long  %01000101

m_LoadPartialPicture    long  %00001101
m_DisplayPartialPicture long  %10001101

m_Count                 long  70000
m_DisplaySize           long  %01000111
m_Reset                 long  %01110111
m_SetDepth              long  %11001111
m_Depth_BlackWhite      long  %00000000
m_Depth_GreyScale       long  %01000000
m_EraseDisplay          long  %11000101
m_CLR_Black             long  %00000000
m_CLR_White             long  %10000000
m_CLR_DarkGray          long  %01000000
m_CLR_LightGray         long  %11000000
m_Orientation           long  %10101111
m_InitDisplay           long  %00100101
m_NegativePicture       long  %00011111
m_Autorefresh               long %10011111 
m_CancelAutorefresh         long %01011111 
m_ManualRefresh             long %00111111

m_Orientation_00        long  %00000000
m_Orientation_90        long  %10000000
m_Orientation_180       long  %01000000
m_Orientation_270       long  %11000000
timer                   long 0


m_NRST       long |<  H_NRST  
m_WUP        long |<  H_WUP  
m_CD         long |<  H_CD
m_DS         long |<  H_DS 
m_RW         long |<  H_RW  
m_ACK        long |<  H_ACK

m_Command               long %0
m_Address               long %0
m_AddressTemp           long %0  
m_X1                    long %0
m_X2                    long %0
m_Y1                    long %0
m_Y2                    long %0
init_par                long 0
m_image_size            long 0
param_idle_wait         long 0
zero                    long 0
m_DataByte               res 1
temp                     res 1
temp2                    res 1
temp_coo                 res 1

fit 496